@@ -15,6 +15,7 @@ from shapely.geometry import Polygon as PolygonS, Point as PointS
 from MJOLNIR import TasUBlibDEG as TasUBlib
 from MJOLNIR._tools import Marray
 from MJOLNIR.Data import Mask
+
 import MJOLNIR.Data.Sample
 import re
 import copy
@@ -781,6 +782,21 @@ class DataFile(object):
             convFile.mask = np.zeros_like(convFile.I,dtype=bool)
             convFile.mask[:,:,:2] = True
 
+
+        if convFile.instrument == 'CAMEA':
+            defectTubes = np.arange(104)[np.any(np.isnan(convFile.instrumentCalibrationA4.reshape(104,-1)),axis=1)] 
+
+            if len(defectTubes)>0: # if any tubes are defect
+                    if len(defectTubes)>1:
+                        warnings.warn('Detector tubes {} masked'.format(', '.join([str(x) for x in defectTubes])))
+                    else:
+                        warnings.warn('Detector tube {} masked'.format(defectTubes[0]))
+
+                    newMask = np.repeat(np.isnan(convFile.instrumentCalibrationA4.reshape(104,-1))[np.newaxis],len(convFile.I),axis=0)
+                    if np.all(convFile.mask.shape==newMask.shape):
+                        newMask = np.logical_or(convFile.mask,newMask)
+                    convFile.mask = newMask
+
         return convFile
 
 
